"""
File:  customerStatesDictionary.py
Author:  Mark Fienup with help from Intro. class
Description:  Reads a small business's customer information from the text
file:  customerData.txt into a customerList.  Each customer record is on
a single line in the text file and generates one entry into the customerList
which happens to be a list of 12 string fields.  The order of the fields are
First Name, Middle Initial, Last Name, Street Address, City, State, Zip Code,
Country, Email Address, Telephone Number, Gender, and Birthday.

The customerList is used to generate a dictionary containing a tally of the
number of customers from each state.

This dictionary is printed in two orders:  sorted by state code, and sorted by
state count.   
"""

def main():
    """ Open's file, reads customer information into a list, closes the file"""
    custFile = open('customerData.txt','r')
    customerList = generateList(custFile)
    custFile.close()
    statesCountDictionary = generateStatesCountDictionary(customerList)
    printSortedByState(statesCountDictionary)
    printSortedByCount(statesCountDictionary)


def generateList(custFile):
    """ Reads customer data from file and returns a list of customers"""
    customers = []
    for line in custFile:
        # Strip the new-line character from the end of the line, then split
        # the line on the commas (',') to get a list of customer fields
        custInfoList = line.strip().split(',')
        customers.append(custInfoList)
    return customers

def generateStatesCountDictionary(customerList):
    """ Tallies the number of customers from each state and
        returns a dictionary with the state as a key and the
        count as the associated value."""
    statesCountDict = {}
    for customer in customerList:
        if customer[5] in statesCountDict:
            count = statesCountDict.get(customer[5])
            count = count + 1
            statesCountDict[customer[5]] = count
        else:
            statesCountDict[customer[5]] = 1
    return statesCountDict

def printSortedByCount(statesCountDictionary):
    """ Prints the tally of the number of customers from each state
        sorted from most to least."""
    stateCountTupleList = statesCountDictionary.items()
    countStateTupleList = []
    for item in stateCountTupleList:
        state, count = item
        countStateTupleList.append((count,state))
    countStateTupleList.sort()
    countStateTupleList.reverse()
    print "\n\n%-14s   %-5s" % ("Customer Count", "State")
    print "-"*25
    for countStateTuple in countStateTupleList:
        count, state = countStateTuple
        print "%9d%12s" % (count, state) 

def printSortedByState(statesCountDictionary):
    """ Prints the tally of the number of customers from each state
        sorted alphabetically by the state abbreviation."""
    tupleList = statesCountDictionary.items()
    tupleList.sort()
    print "\n\n%5s   %-14s" % ("State","Customer Count")
    print "-"*25
    for item in tupleList:
        state, count = item
        print "%5s   %8d" % (state.center(5), count) 

main()
